package net.hoosiertransfer.EaglerXMicrosoft.network;

import org.json.JSONObject;
import org.teavm.interop.Async;
import org.teavm.interop.AsyncCallback;
import org.teavm.jso.JSBody;
import org.teavm.jso.ajax.XMLHttpRequest;

import java.util.HashMap;
import java.util.Map;

public class PostRequest {
    public Map<String, String> headers;
    public Map<String, String> contents;

    public PostRequest() {
        headers = new HashMap<>();
        contents = new HashMap<>();

        headers.put("Content-Type", "application/x-www-form-urlencoded");
    }

    public void addHeader(String key, String value) {
        headers.put(key, value);
    }

    public void removeHeader(String key) {
        headers.remove(key);
    }

    public void put(String key, String value) {
        contents.put(key, value);
    }

    @Async
    public native JSONObject send(String url);

    private void send(String url, AsyncCallback<JSONObject> callback) {
        XMLHttpRequest xhr = XMLHttpRequest.create();
        String corsProxyUrl = "https://corsproxy.io/?" + encodeURIComponent(url);
        xhr.open("POST", corsProxyUrl);
        xhr.setRequestHeader("Access-Control-Allow-Origin", "*");
        for (Map.Entry<String, String> entry : headers.entrySet()) {
            xhr.setRequestHeader(entry.getKey(), entry.getValue());
        }
        StringBuilder bodyBuilder = new StringBuilder();
        for (Map.Entry<String, String> entry : contents.entrySet()) {
            bodyBuilder.append(entry.getKey()).append("=").append(entry.getValue()).append("&");
        }
        String body = bodyBuilder.substring(0, bodyBuilder.length() - 1);

        xhr.setOnReadyStateChange(() -> {
            if (xhr.getReadyState() == XMLHttpRequest.DONE) {
                if (xhr.getStatus() == 200) {
                    callback.complete(new JSONObject(xhr.getResponseText()));
                    if (xhr.getStatus() == 400) {
                        callback.error(new Exception("HTTP error " + xhr.getStatus() + ": " + xhr.getResponseText()));
                    } else {
                        callback.error(new Exception("HTTP error " + xhr.getStatus()));
                    }
                }
            }
        });
        xhr.send(body);
    }

    @JSBody(params = "url", script = "return encodeURIComponent(url);")
    private static native String encodeURIComponent(String url);
}
